package warehouse.ejb;
import javax.ejb.*;
import java.sql.*;
import javax.sql.*;
import javax.naming.*;
import com.sun.jts.utils.RecoveryHooks.FailureInducer;

@Stateful
public class WarehouseBean implements Warehouse {
	
    private static final String dbName1 = "jdbc/WarehouseDB1";
	private static final String dbName2 = "jdbc/WarehouseDB2";

    private Connection con1;
    private Connection con2;


    public WarehouseBean() {
    }

    public void ship(String productId, String orderId, int quantity) {
        try {
        	FailureInducer.activateFailureInducer();
        	
            makeConnection1();
            makeConnection2();
            
            updateOrderItemForDB1(productId, orderId);
            updateInventoryForDB1(productId, quantity);
            updateOrderItemForDB2(productId, orderId);
            updateInventoryForDB2(productId, quantity);

            //System.out.print("Setting wait point for 60  seconds at PREPARED");
   			FailureInducer.setWaitPoint(FailureInducer.PREPARED, 60);
                       
        } catch (Exception ex) {
           	//StackTraceElement[] stackTrace=null;
			ex.printStackTrace();
            try {
                con1.rollback();
                throw new EJBException("Transaction failed: " +
                    ex.getMessage(),ex);
            } catch (SQLException sqx) {
                throw new EJBException("Rollback failed: " + sqx.getMessage(),ex);
            }
        } finally {
            releaseConnection1();
            releaseConnection2();
        }
    }

    public String getStatusForDB1(String productId, String orderId) {
        try {
            makeConnection1();

            return selectStatusForDB1(productId, orderId);
        } catch (SQLException ex) {
            throw new EJBException(
                "Unable to fetch status due to SQLException: " +
                ex.getMessage());
        } finally {
            releaseConnection1();
        }
    }
  
    public String getStatusForDB2(String productId, String orderId) {
        try {
            makeConnection2();

            return selectStatusForDB2(productId, orderId);
        } catch (SQLException ex) {
            throw new EJBException(
                "Unable to fetch status due to SQLException: " +
                ex.getMessage());
        } finally {
            releaseConnection2();
        }
    }
    
    @Remove()
    public void remove() {
    }
    

    /********************* Database Routines ***********************/
    private String selectStatusForDB1(String productId, String orderId)
        throws SQLException {
        String result;

        String selectStatement =
            "select status " + "from order_item where product_id = ? " +
            "and order_id = ?";
        PreparedStatement prepStmt = con1.prepareStatement(selectStatement);

        prepStmt.setString(1, productId);
        prepStmt.setString(2, orderId);

        ResultSet rs = prepStmt.executeQuery();

        if (rs.next()) {
            result = rs.getString(1);
        } else {
            result = "No rows found.";
        }

        prepStmt.close();

        return result;
    }

    private String selectStatusForDB2(String productId, String orderId)
    throws SQLException {
    String result;

    String selectStatement =
        "select status " + "from order_item where product_id = ? " +
        "and order_id = ?";
    PreparedStatement prepStmt = con2.prepareStatement(selectStatement);

    prepStmt.setString(1, productId);
    prepStmt.setString(2, orderId);

    ResultSet rs = prepStmt.executeQuery();

    if (rs.next()) {
        result = rs.getString(1);
    } else {
        result = "No rows found.";
    }

    prepStmt.close();

    return result;
}
    
    private void updateOrderItemForDB1(String productId, String orderId)
        throws SQLException {
        String updateStatement =
            "update order_item set status =  'shipped' " +
            "where product_id = ? " + "and order_id = ?";

        PreparedStatement prepStmt = con1.prepareStatement(updateStatement);

        prepStmt.setString(1, productId);
        prepStmt.setString(2, orderId);
        prepStmt.executeUpdate();
        prepStmt.close();
    }

    private void updateInventoryForDB1(String productId, int quantity)
        throws SQLException {
        String updateStatement =
            "update inventory " + "set quantity = quantity - ? " +
            "where product_id = ?";

        PreparedStatement prepStmt = con1.prepareStatement(updateStatement);

        prepStmt.setInt(1, quantity);
        prepStmt.setString(2, productId);
        prepStmt.executeUpdate();
        prepStmt.close();
    }

	private void updateOrderItemForDB2(String productId, String orderId)
	    throws SQLException {
	    String updateStatement =
	        "update order_item set status =  'shipped' " +
	        "where product_id = ? " + "and order_id = ?";
	
	    PreparedStatement prepStmt = con2.prepareStatement(updateStatement);
	
	    prepStmt.setString(1, productId);
	    prepStmt.setString(2, orderId);
	    prepStmt.executeUpdate();
	    prepStmt.close();
	}

	private void updateInventoryForDB2(String productId, int quantity)
	    throws SQLException {
	    String updateStatement =
	        "update inventory " + "set quantity = quantity - ? " +
	        "where product_id = ?";
	
	    PreparedStatement prepStmt = con2.prepareStatement(updateStatement);
	
	    prepStmt.setInt(1, quantity);
	    prepStmt.setString(2, productId);
	    prepStmt.executeUpdate();
	    prepStmt.close();
	}
    
    
    private void makeConnection1() {
        try {
            InitialContext ic1 = new InitialContext();
            DataSource ds1 = (DataSource) ic1.lookup(dbName1);

            con1 = ds1.getConnection();
        } catch (Exception ex) {
            throw new EJBException("Unable to connect to database 1. " +
                ex.getMessage());
        }
    }

    private void makeConnection2() {
        try {
            InitialContext ic2 = new InitialContext();
            DataSource ds2 = (DataSource) ic2.lookup(dbName2);
            con2 = ds2.getConnection();
        } catch (Exception ex) {
            throw new EJBException("Unable to connect to database 2. " +
                ex.getMessage());
        }
    }

    private void releaseConnection1() {
        try {
            con1.close();
        } catch (SQLException ex) {
            throw new EJBException("releaseConnection 1: " + ex.getMessage());
        }
    }
    
    private void releaseConnection2() {
        try {
            con2.close();
        } catch (SQLException ex) {
            throw new EJBException("releaseConnection 2: " + ex.getMessage());
        }
    }
} // WarehouseBean 
